import {IncomingMessage} from 'http';
import {MediaType, Charset} from '../common';
import {BaseSchema, parseAsync} from 'valibot';

export const getBody = (
    req: IncomingMessage,
    schema: BaseSchema,
    encodingPair?: Charset,
    deserializer?: MediaType,
) => new Promise((resolve, reject) => {
    let body = Buffer.from('');
    req.on('data', (chunk) => {
        body = Buffer.concat([body, chunk]);
    });
    req.on('end', async () => {
        const bodyStr = encodingPair?.decode(body) ?? body.toString();
        try {
            const bodyDeserialized = await parseAsync(
                schema,
                deserializer?.deserialize(bodyStr) ?? body,
                {abortEarly: false},
            );
            resolve(bodyDeserialized);
        } catch (err) {
            reject(err);
        }
    });
});
