import * as applicationJson from '../common/media-types/application/json';
import * as utf8 from '../common/charsets/utf-8';
import * as en from '../common/languages/en';
import {ApplicationState, Charset, Language, MediaType} from '../common';

export interface ClientState {
	app: ApplicationState;
	mediaType: MediaType;
	charset: Charset;
	language: Language;
}

export interface ClientBuilder {
	setLanguage(languageCode: ClientState['language']['name']): this;
	setCharset(charset: ClientState['charset']['name']): this;
	setMediaType(mediaType: ClientState['mediaType']['name']): this;
}

export interface CreateClientParams {
	app: ApplicationState;
}

export const createClient = (params: CreateClientParams) => {
	const clientState: ClientState = {
		app: params.app,
		mediaType: applicationJson,
		charset: utf8,
		language: en
	};

	return {
		setMediaType(mediaTypeName) {
			const mediaType = Array.from(clientState.app.mediaTypes).find((l) => l.name === mediaTypeName);
			clientState.mediaType = mediaType ?? applicationJson;
			return this;
		},
		setCharset(charsetName) {
			const charset = Array.from(clientState.app.charsets).find((l) => l.name === charsetName);
			clientState.charset = charset ?? utf8;
			return this;
		},
		setLanguage(languageCode) {
			const language = Array.from(clientState.app.languages).find((l) => l.name === languageCode);
			clientState.language = language ?? en;
			return this;
		}
	} satisfies ClientBuilder;
};
